/* packet-ldss.c
 * Routines for Local Download Sharing Service dissection
 * Copyright 2006, Vasantha Crabb <vcrabb@managesoft.com.au>
 *
 * $Id: packet-ldss.c,v 1.1.4.1.2.1 2006/05/26 05:43:13 vcrabb Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * Copied from WHATEVER_FILE_YOU_USED (where "WHATEVER_FILE_YOU_USED"
 * is a dissector file; if you just copied this from README.developer,
 * don't bother with the "Copied from" - you don't even need to put
 * in a "Copied from" if you copied an existing dissector, especially
 * if the bulk of the code in the new dissector is your code)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Include files */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "moduleinfo.h"

#include <stdio.h>
#include <stdlib.h>
#include <gmodule.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <epan/packet.h>
#include <epan/addr_resolv.h>
#include <epan/prefs.h>
#include <epan/strutil.h>

/* Define version if we are not building ethereal statically */

#ifndef ENABLE_STATIC
G_MODULE_EXPORT const gchar version[] = VERSION;
#endif


#define MESSAGE_ID_NEEDFILE	0
#define MESSAGE_ID_WILLSEND	1


const value_string ldss_message_id_value[] = {
    { MESSAGE_ID_NEEDFILE,	"Need file"	},
    { MESSAGE_ID_WILLSEND,	"Will send"	},
    { 0,			NULL		}
};


#define DIGEST_TYPE_UNKNOWN	0
#define DIGEST_TYPE_MD5		1
#define DIGEST_TYPE_SHA1	2
#define DIGEST_TYPE_SHA256	3


const value_string ldss_digest_type_value[] = {
    { DIGEST_TYPE_UNKNOWN,	"Unknown"	},
    { DIGEST_TYPE_MD5,		"MD5"		},
    { DIGEST_TYPE_SHA1,		"SHA1"		},
    { DIGEST_TYPE_SHA256,	"SHA256"	},
    { 0,			NULL		}
};


#define COMPRESSION_NONE	0
#define COMPRESSION_GZIP	1


const value_string ldss_compression_value[] = {
    { COMPRESSION_NONE,		"None"		},
    { COMPRESSION_GZIP,		"gzip"		},
    { 0,			NULL		}
};


/* Define udp_port for LDSS */

#define UDP_PORT_LDSS 6087

void proto_reg_handoff_ldss(void);

/* Define the ldss proto */

static int	proto_ldss		= -1;

/* Define headers for ldss */

static int	hf_ldss_message_id	= -1;
static int	hf_ldss_digest_type	= -1;
static int	hf_ldss_compression	= -1;
static int	hf_ldss_cookie		= -1;
static int	hf_ldss_digest		= -1;
static int	hf_ldss_size		= -1;
static int	hf_ldss_offset		= -1;
static int	hf_ldss_target_time	= -1;
static int	hf_ldss_reserved_1	= -1;
static int	hf_ldss_port		= -1;
static int	hf_ldss_rate		= -1;
static int	hf_ldss_priority	= -1;
static int	hf_ldss_property_count	= -1;
static int	hf_ldss_properties	= -1;

/* Define the tree for ldss */

static int	ett_ldss		= -1;

/* 
 * Here are the global variables associated with the preferences
 * for ldss
 */

static guint	global_udp_port_ldss	= UDP_PORT_LDSS;
static guint	udp_port_ldss		= UDP_PORT_LDSS;


static void 
dissect_ldss (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
  
    guint16	messageID;
    guint64	size;
    guint64	offset;
    guint32	targetTime;
    guint16	rate;

    proto_tree	*ti, *ldss_tree;

    /* Set the protocol column */
    if(check_col(pinfo->cinfo,COL_PROTOCOL)){
	col_add_str(pinfo->cinfo,COL_PROTOCOL,"LDSS");
    }

    /* Get the stuff */
    messageID	= tvb_get_ntohs  (tvb,  0);
    size	= tvb_get_ntoh64 (tvb,  40);
    offset	= tvb_get_ntoh64 (tvb,  48);
    targetTime	= tvb_get_ntohl  (tvb,  56);
    rate	= tvb_get_ntohs  (tvb,  66);

    if (check_col(pinfo->cinfo, COL_INFO)) 
	col_add_fstr(	pinfo->cinfo, COL_INFO, "LDSS Packet (%s%s)",
			val_to_str(messageID, ldss_message_id_value, "unknown"),
			(messageID == MESSAGE_ID_WILLSEND)
			    ? ((size > 0)
				? ((size == offset)
				    ? " - offer"
				    : " - promise")
				: "")
			    : "");

    /* 
     * If we have a non-null tree (ie we are building the proto_tree
     * instead of just filling out the columns ), then add a PLCI
     * tree node and put a CCCID header element under it.
     */
    if (tree) {
	ti = proto_tree_add_item(	tree, proto_ldss,
					tvb, 0, (tvb_length(tvb) > 72) ? tvb_length(tvb) : 72, FALSE);
	ldss_tree = proto_item_add_subtree(ti, ett_ldss);

	ti = proto_tree_add_item(		ldss_tree, hf_ldss_message_id,
						tvb, 0,	    2,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_digest_type,
						tvb, 2,	    1,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_compression,
						tvb, 3,	    1,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_cookie,
						tvb, 4,	    4,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_digest,
						tvb, 8,	    32, FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_size,
						tvb, 40,    8,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_offset,
						tvb, 48,    8,	FALSE);
	ti = proto_tree_add_uint_format(	ldss_tree, hf_ldss_target_time,
						tvb, 56,    4,	FALSE,
						"%s: %d:%02d:%02d",
						"Target time (relative)",
						(int)(targetTime / 3600),
						(int)((targetTime / 60) % 60),
						(int)(targetTime % 60));
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_reserved_1,
						tvb, 60,    4,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_port,
						tvb, 64,    2,	FALSE);
	ti = proto_tree_add_uint_format(	ldss_tree, hf_ldss_rate,
						tvb, 66,    2,	FALSE,
						"%s: %ld",
						"Rate (B/s)",
						(rate > 0)
						    ? (long)floor(exp(rate * G_LN2 / 2048))
						    : 0);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_priority,
						tvb, 68,    2,	FALSE);
	ti = proto_tree_add_item(		ldss_tree, hf_ldss_property_count,
						tvb, 70,    2,	FALSE);
	if (tvb_length(tvb) > 72)
	    ti = proto_tree_add_item(		ldss_tree, hf_ldss_properties,
						tvb, 72,    tvb_length(tvb) - 72, FALSE);
    }
}

void 
proto_register_ldss (void) {
    static hf_register_info hf[] =
	    {	{   &hf_ldss_message_id,
		    {	"LDSS Message ID",
			"ldss.ldss_message_id",
			FT_UINT16, BASE_DEC, ldss_message_id_value, 0x0,
			"", HFILL
		    }
		},
		{   &hf_ldss_digest_type,
		    {	"Digest Type",
			"ldss.digest_type",
			FT_UINT8, BASE_DEC, ldss_digest_type_value, 0x0,	  
			"", HFILL
		    }
		},
		{   &hf_ldss_compression,
		    {	"Compressed Format",
			"ldss.compression",
			FT_UINT8, BASE_DEC, ldss_compression_value, 0x0,	  
			"", HFILL
		    }
		},
		{   &hf_ldss_cookie,
		    {	"Cookie",
			"ldss.cookie",
			FT_UINT32, BASE_HEX, NULL, 0x0,		 
			"Random value used for duplicate rejection", HFILL
		    }
		},
		{   &hf_ldss_digest,
		    {	"Digest",
			"ldss.digest",
			FT_BYTES, BASE_HEX, NULL, 0x0,		
			"Digest of file padded with 0x00", HFILL
		    }
		},
		{   &hf_ldss_size,
		    {	"Size",
			"ldss.size",
			FT_UINT64, BASE_DEC, NULL, 0x0,		 
			"Size of file complete file", HFILL
		    }
		},
		{   &hf_ldss_offset,
		    {	"Offset",
			"ldss.offset",
			FT_UINT64, BASE_DEC, NULL, 0x0,		 
			"Size of currently available portion of file", HFILL
		    }
		},
		{   &hf_ldss_target_time,
		    {	"Target time (relative)",
			"ldss.target_time",
			FT_UINT32, BASE_DEC, NULL, 0x0,		 
			"Time until file will be needed/available", HFILL
		    }
		},
		{   &hf_ldss_reserved_1,
		    {	"Reserved",
			"ldss.reserved_1",
			FT_UINT32, BASE_HEX, NULL, 0x0,		 
			"Unused field - should be 0x00000000", HFILL
		    }
		},
		{   &hf_ldss_port,
		    {	"Port",
			"ldss.port",
			FT_UINT16, BASE_DEC, NULL, 0x0,		 
			"TCP port for push (Need file) or pull (Will send)", HFILL
		    }
		},
		{   &hf_ldss_rate,
		    {	"Rate (B/s)",
			"ldss.rate",
			FT_UINT16, BASE_DEC, NULL, 0x0,		 
			"Estimated current download rate", HFILL
		    }
		},
		{   &hf_ldss_priority,
		    {	"Priority",
			"ldss.priority",
			FT_UINT16, BASE_DEC, NULL, 0x0,		 
			"", HFILL
		    }
		},
		{   &hf_ldss_property_count,
		    {	"Property Count",
			"ldss.property_count",
			FT_UINT16, BASE_DEC, NULL, 0x0,		 
			"", HFILL
		    }
		},
		{   &hf_ldss_properties,
		    {	"Properties:",
			"ldss.properties",
			FT_BYTES, BASE_HEX, NULL, 0x0,		
			"", HFILL
		    }
		}
	    };

    static gint		    *ett[] = { &ett_ldss };

    module_t		    *ldss_module;

    proto_ldss = proto_register_protocol(	"Local Download Sharing Service",
						"LDSS", "ldss");
    proto_register_field_array(proto_ldss, hf, array_length(hf));
    proto_register_subtree_array(ett, array_length(ett));

    ldss_module = prefs_register_protocol(	proto_ldss,
						proto_reg_handoff_ldss);
    prefs_register_uint_preference(		ldss_module, "udp_port",
						"LDSS UDP Port",
						"The UDP port on which "
						"Local Download Sharing Service "
						"packets will be sent",
						10, &global_udp_port_ldss);

}

/* The registration hand-off routing */

void
proto_reg_handoff_ldss (void) {
    static int			ldss_initialized	= FALSE;
    static dissector_handle_t	ldss_handle;

    if (!ldss_initialized) {
	ldss_handle = create_dissector_handle(dissect_ldss, proto_ldss);
	ldss_initialized = TRUE;
    } else {
	dissector_delete("udp.port", udp_port_ldss, ldss_handle);
    }

    udp_port_ldss = global_udp_port_ldss;

    dissector_add("udp.port", global_udp_port_ldss, ldss_handle);
}

/* Start the functions we need for the plugin stuff */

#ifndef ENABLE_STATIC

G_MODULE_EXPORT void
plugin_register (void)
{
    /* register the new protocol, protocol fields, and subtrees */
    if (proto_ldss == -1) { /* execute protocol initialization only once */
	proto_register_ldss();
    }
}

G_MODULE_EXPORT void
plugin_reg_handoff (void) {
    proto_reg_handoff_ldss();
}

#endif

/* End the functions we need for plugin stuff */

